
# q-expansion of newform 1323.2.a.be, downloaded from the LMFDB on 19 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1323
weight = 2
poly_data = [4, 0, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, -4, 0, 1], 2], [[-3, 0, 1, 0], 1]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0, 0], [0, 0, 0, 0], [2, 0, 0, 1], [0, 0, 0, 0], [0, -1, -1, 0], [0, -1, 0, 0], [5, 0, 0, 0], [0, 3, -1, 0], [0, 2, -3, 0], [0, -5, 3, 0], [0, 1, -4, 0], [2, 0, 0, -3], [2, 0, 0, -3], [0, 0, 0, 1], [3, 0, 0, -2], [0, 2, -1, 0], [5, 0, 0, -4], [0, 1, 1, 0], [-9, 0, 0, -1], [0, 2, 6, 0], [0, -4, 7, 0], [-7, 0, 0, -2], [2, 0, 0, -1], [5, 0, 0, -3], [0, -1, 3, 0], [7, 0, 0, 1], [0, -8, 7, 0], [0, 0, -7, 0], [12, 0, 0, -1], [0, 3, -4, 0], [1, 0, 0, 2], [2, 0, 0, 4], [0, -3, 8, 0], [0, -9, 1, 0], [0, 9, 1, 0], [-3, 0, 0, -6], [0, -1, -13, 0], [6, 0, 0, -5], [-6, 0, 0, 1], [14, 0, 0, 2], [0, -1, 12, 0], [0, 10, 0, 0], [0, 0, -3, 0], [6, 0, 0, 5], [0, 7, -11, 0], [0, -4, 16, 0], [-4, 0, 0, -6], [0, -11, 4, 0], [1, 0, 0, 5], [0, -4, 14, 0], [0, -5, 8, 0], [0, -5, -10, 0], [0, 3, 0, 0], [-16, 0, 0, 3], [8, 0, 0, -6], [0, -15, 10, 0], [7, 0, 0, 0], [0, 8, -9, 0], [-16, 0, 0, 1], [0, 2, -19, 0], [0, 9, -3, 0], [3, 0, 0, 2], [0, 8, -11, 0], [-2, 0, 0, 13], [0, -4, -14, 0], [0, -4, 5, 0], [5, 0, 0, 6], [6, 0, 0, -1], [0, -1, 18, 0], [0, 6, -2, 0], [10, 0, 0, 9], [0, 6, -13, 0], [0, 10, -3, 0], [2, 0, 0, -7], [-13, 0, 0, -8], [0, 0, 0, -5], [0, 7, 5, 0], [0, 14, -3, 0], [0, -2, 20, 0], [0, 14, 0, 0], [0, 0, 0, 7], [0, 0, 0, -2], [0, 10, -2, 0], [0, -17, 16, 0], [0, 6, -10, 0], [0, 15, -9, 0], [0, -9, -10, 0], [6, 0, 0, 12], [27, 0, 0, -2], [26, 0, 0, 5], [-13, 0, 0, -11], [-2, 0, 0, -13], [-8, 0, 0, -8], [0, -13, 20, 0], [19, 0, 0, 8], [9, 0, 0, 12], [5, 0, 0, -10], [-13, 0, 0, 2], [0, -13, 18, 0], [-3, 0, 0, 14], [-2, 0, 0, 9], [0, 4, -2, 0], [23, 0, 0, 5], [0, 8, 5, 0], [10, 0, 0, -13], [0, 18, -2, 0], [-14, 0, 0, -2], [7, 0, 0, 2], [0, 11, 5, 0], [0, 5, -26, 0], [0, 3, 14, 0], [-14, 0, 0, -8], [0, 4, 1, 0], [0, -15, -3, 0], [-24, 0, 0, -1], [0, -1, -23, 0], [0, -8, -11, 0], [18, 0, 0, -10], [0, -13, 8, 0], [0, 2, 24, 0], [0, -19, 10, 0], [-39, 0, 0, -1], [30, 0, 0, -8], [0, 17, 0, 0], [0, 9, -14, 0], [0, -7, 18, 0], [-17, 0, 0, -2], [-13, 0, 0, 6], [0, 5, 17, 0], [0, 4, -2, 0], [-26, 0, 0, 10], [0, 7, 7, 0], [29, 0, 0, 7], [-13, 0, 0, -10], [-31, 0, 0, -2], [0, 6, -30, 0], [24, 0, 0, -11], [0, -12, 17, 0], [13, 0, 0, -13], [0, -4, -1, 0], [0, 3, 22, 0], [0, -14, 14, 0], [-23, 0, 0, 6], [0, -15, -4, 0], [0, 3, -21, 0], [-36, 0, 0, 5], [0, -18, 14, 0], [8, 0, 0, -5], [0, -14, -11, 0], [0, 12, 1, 0], [-12, 0, 0, -17], [-10, 0, 0, 12], [-1, 0, 0, 8], [3, 0, 0, 22], [47, 0, 0, 0], [0, -3, 21, 0], [-7, 0, 0, 4], [9, 0, 0, -12], [0, 1, 11, 0], [14, 0, 0, 3], [0, 24, -10, 0], [0, 4, -17, 0], [10, 0, 0, -2], [-25, 0, 0, 2], [0, -12, 16, 0], [-35, 0, 0, 4], [-20, 0, 0, 9], [0, 25, 3, 0], [-10, 0, 0, -3], [-10, 0, 0, 13], [0, -14, 16, 0], [0, -17, -1, 0], [0, -11, 18, 0], [-39, 0, 0, 8], [0, -1, 1, 0], [3, 0, 0, 8], [0, 0, 0, -14], [0, 22, -28, 0], [0, 12, -30, 0], [0, 10, -7, 0], [4, 0, 0, -1], [-4, 0, 0, 15], [-16, 0, 0, 2], [24, 0, 0, -1], [0, -5, 17, 0], [5, 0, 0, 12], [-18, 0, 0, -4], [0, -7, -2, 0], [7, 0, 0, 3], [-18, 0, 0, 3], [0, -23, 16, 0], [0, -4, -8, 0], [-16, 0, 0, 7], [-31, 0, 0, 10], [0, -12, 18, 0], [-48, 0, 0, -1], [0, 0, 0, 14], [-40, 0, 0, -11], [6, 0, 0, 14], [-8, 0, 0, -11], [0, 6, -10, 0], [0, -6, 30, 0], [0, 24, 1, 0], [0, -30, 27, 0], [-26, 0, 0, -11], [13, 0, 0, 14], [0, 7, 3, 0], [0, 14, -2, 0], [0, 34, -27, 0], [0, 15, 2, 0], [9, 0, 0, 6], [16, 0, 0, -12], [-10, 0, 0, 13], [-20, 0, 0, -9], [-27, 0, 0, -14], [0, -15, 29, 0], [19, 0, 0, -16], [-45, 0, 0, -5], [0, 5, 4, 0], [0, -28, -6, 0], [-16, 0, 0, -3], [0, -35, 12, 0], [0, -23, -12, 0], [51, 0, 0, 4], [-32, 0, 0, -1], [-8, 0, 0, 15], [-16, 0, 0, -7], [0, -36, 15, 0], [0, -20, -5, 0], [0, 13, 18, 0], [30, 0, 0, 0], [0, -11, -22, 0], [-24, 0, 0, 15], [0, 16, -16, 0], [0, -24, 31, 0], [8, 0, 0, 14], [0, -8, 10, 0], [0, 7, 9, 0], [0, 12, 1, 0], [-9, 0, 0, 4], [0, 8, -20, 0], [0, 26, -10, 0], [0, 15, -8, 0], [28, 0, 0, 6], [9, 0, 0, -15], [6, 0, 0, 3], [0, -16, -13, 0], [-19, 0, 0, 10], [-9, 0, 0, 24], [0, -22, 1, 0], [33, 0, 0, 10], [-24, 0, 0, 10], [0, -32, 3, 0], [0, -16, -5, 0], [22, 0, 0, -17], [0, -11, -25, 0], [32, 0, 0, -15], [0, 28, -5, 0], [49, 0, 0, -3], [0, 6, -25, 0], [41, 0, 0, -5], [0, -10, -23, 0], [0, 2, -16, 0], [0, 7, 5, 0], [-19, 0, 0, -5], [0, 2, 12, 0], [0, -18, 40, 0], [-40, 0, 0, -4], [11, 0, 0, -12], [0, -10, -17, 0], [0, -5, 16, 0], [-47, 0, 0, 7], [0, 7, 4, 0], [-43, 0, 0, 6], [0, 8, 31, 0], [0, 1, 13, 0], [0, -15, -4, 0], [-30, 0, 0, 13], [23, 0, 0, -10], [34, 0, 0, 19], [31, 0, 0, -6], [-46, 0, 0, 1], [13, 0, 0, -24], [0, 6, -44, 0], [0, 0, -37, 0], [0, 4, -30, 0], [39, 0, 0, 0], [0, -19, 41, 0], [0, 26, -25, 0], [1, 0, 0, -36], [0, 8, -33, 0], [6, 0, 0, -7], [0, -15, 31, 0], [23, 0, 0, -14], [-53, 0, 0, -6], [79, 0, 0, 2], [0, -2, 39, 0], [58, 0, 0, 3], [25, 0, 0, -8], [0, -47, 17, 0], [0, -7, -16, 0], [0, 0, -14, 0], [6, 0, 0, -10], [0, -43, 16, 0], [-29, 0, 0, 8], [13, 0, 0, 19], [0, -12, 24, 0], [0, -2, 11, 0], [0, -5, -14, 0], [25, 0, 0, -7], [15, 0, 0, 1], [0, -33, 22, 0], [0, -14, 17, 0], [15, 0, 0, 10], [0, 2, -4, 0], [0, -48, 23, 0], [46, 0, 0, 15], [0, -10, -23, 0], [0, 25, -13, 0], [0, 14, -6, 0], [0, 20, -27, 0], [-7, 0, 0, -26], [24, 0, 0, 7], [10, 0, 0, -18], [0, 7, 8, 0], [0, 20, -45, 0], [-1, 0, 0, 12], [0, 46, -30, 0], [0, 21, -35, 0], [0, -3, 7, 0], [2, 0, 0, -8], [0, 11, 13, 0], [0, 3, 27, 0], [19, 0, 0, -6], [-39, 0, 0, -4], [-47, 0, 0, -10], [27, 0, 0, -11], [48, 0, 0, 5], [0, 12, 30, 0], [0, -10, -12, 0], [-14, 0, 0, 5], [0, -19, -27, 0], [-10, 0, 0, 23], [10, 0, 0, -20], [0, 18, -37, 0], [0, -27, 33, 0], [0, -38, 0, 0], [-23, 0, 0, -22], [8, 0, 0, 9], [0, 0, 0, -14], [0, -7, -20, 0], [-3, 0, 0, -26], [0, 2, -7, 0], [0, -23, 25, 0], [-16, 0, 0, 29], [16, 0, 0, 17], [32, 0, 0, -8], [58, 0, 0, -15], [0, -6, -4, 0], [0, -5, -21, 0], [17, 0, 0, -10], [48, 0, 0, 1], [0, -21, 40, 0], [0, -24, 18, 0], [0, 18, 14, 0], [-42, 0, 0, 15], [42, 0, 0, -9], [1, 0, 0, -10], [-39, 0, 0, 2], [0, -16, 2, 0], [0, -45, 18, 0], [0, 17, -5, 0], [0, -11, 14, 0], [0, -1, -10, 0], [28, 0, 0, 4], [-40, 0, 0, -16], [0, 11, -54, 0], [0, 5, -44, 0], [-27, 0, 0, 7], [0, 34, -12, 0], [-20, 0, 0, -11], [0, -7, -59, 0], [-12, 0, 0, 16], [0, -26, 17, 0], [0, -20, -31, 0], [-74, 0, 0, 0], [43, 0, 0, 15], [0, 11, 12, 0], [16, 0, 0, 13], [39, 0, 0, 11], [-49, 0, 0, -12], [16, 0, 0, -30], [0, -20, 57, 0], [0, 17, 0, 0], [0, -49, 27, 0], [-32, 0, 0, 2], [0, -8, 27, 0], [-57, 0, 0, 10], [-46, 0, 0, 14], [0, 14, 23, 0], [0, 2, -4, 0], [0, 28, -14, 0], [-12, 0, 0, -7], [22, 0, 0, 35], [-36, 0, 0, -18], [0, -3, -29, 0], [-6, 0, 0, 1], [0, 14, 5, 0], [0, -30, 40, 0], [-19, 0, 0, 5], [0, 22, 20, 0], [0, -27, -2, 0], [0, 45, -30, 0], [93, 0, 0, 2], [6, 0, 0, 29], [0, 0, 0, -2], [0, 5, 8, 0], [0, 21, -35, 0], [-50, 0, 0, 15], [9, 0, 0, 27], [0, -9, -20, 0], [0, 33, 15, 0], [0, 25, -32, 0], [-5, 0, 0, 26], [0, 41, 10, 0], [7, 0, 0, -36], [0, 6, 40, 0], [0, 21, -27, 0], [0, 16, -19, 0], [12, 0, 0, -6]]
